@@ -1,9 +1,10 @@
-__all__ = ["Config", "SmartPortConfig", "RegisterAccess", "SIGENERGY_MODBUS_PROTOCOL", "SIGENERGY_MODBUS_PROTOCOL_PUBLISHED"]
+__all__ = ["Config", "SmartPortConfig", "RegisterAccess", "SIGENERGY_MODBUS_PROTOCOL", "SIGENERGY_MODBUS_PROTOCOL_PUBLISHED", "CONSUMPTION", "IMPORTED", "OutputField", "StatusField"]
 
 
 from . import const
 from .config import Config
 from .modbus_config import RegisterAccess, SmartPortConfig
+from .pvoutput_config import CONSUMPTION, IMPORTED, OutputField, StatusField
 from .version import SIGENERGY_MODBUS_PROTOCOL, SIGENERGY_MODBUS_PROTOCOL_PUBLISHED
 from pathlib import Path
 import argparse
@@ -161,6 +162,15 @@ _parser.add_argument(
     default=os.getenv(const.SIGENERGY2MQTT_MQTT_PORT, None),
     help="The listening port of the MQTT broker (default is 1883, unless --mqtt-tls is specified, in which case the default is 8883)",
 )
+_parser.add_argument(
+    "--mqtt-keepalive",
+    nargs="?",
+    action="store",
+    dest=const.SIGENERGY2MQTT_MQTT_KEEPALIVE,
+    type=int,
+    default=os.getenv(const.SIGENERGY2MQTT_MQTT_KEEPALIVE, None),
+    help="The maximum period in seconds between communications with the broker. If no other messages are being exchanged, this controls the rate at which the client will send ping messages to the broker. Default is 60 and minimum is 1.",
+)
 _parser.add_argument(
     "--mqtt-tls",
     action="store_true",
@@ -247,7 +257,7 @@ _parser.add_argument(
     dest=const.SIGENERGY2MQTT_MODBUS_INVERTER_DEVICE_ID,
     type=int,
     default=os.getenv(const.SIGENERGY2MQTT_MODBUS_INVERTER_DEVICE_ID, None),
-    help="**The Sigenergy Inverter Modbus Device ID. May be specified multiple times.",
+    help="**The Sigenergy Inverter Modbus Device ID. Multiple device IDS may be specified, separated by spaces.",
 )
 _parser.add_argument(
     "--modbus-accharger-slave",
@@ -295,6 +305,30 @@ _parser.add_argument(
     dest=const.SIGENERGY2MQTT_MODBUS_NO_REMOTE_EMS,
     help="Do not publish any read-write sensors for remote Energy Management System (EMS) integration to MQTT. Ignored if --modbus-read-only is specified.",
 )
+_parser.add_argument(
+    "--modbus-timeout",
+    nargs="?",
+    action="store",
+    dest=const.SIGENERGY2MQTT_MODBUS_TIMEOUT,
+    type=float,
+    default=os.getenv(const.SIGENERGY2MQTT_MODBUS_TIMEOUT, None),
+    help="The timeout for connecting and receiving Modbus data, in seconds (use decimals for milliseconds). The default is 1.0.",
+)
+_parser.add_argument(
+    "--modbus-retries",
+    nargs="?",
+    action="store",
+    dest=const.SIGENERGY2MQTT_MODBUS_RETRIES,
+    type=int,
+    default=os.getenv(const.SIGENERGY2MQTT_MODBUS_RETRIES, None),
+    help="The maximum number of times to retry a Modbus operation if it fails. The default is 3.",
+)
+_parser.add_argument(
+    "--modbus-disable-chunking",
+    action="store_true",
+    dest=const.SIGENERGY2MQTT_MODBUS_DISABLE_CHUNKING,
+    help="Disable Modbus chunking when reading registers and read each register individually.",
+)
 _parser.add_argument(
     "--modbus-log-level",
     action="store",
@@ -312,7 +346,7 @@ _parser.add_argument(
     dest=const.SIGENERGY2MQTT_SCAN_INTERVAL_LOW,
     type=int,
     default=os.getenv(const.SIGENERGY2MQTT_SCAN_INTERVAL_LOW, None),
-    help="The scan interval in seconds for Modbus registers that are to be scanned at a low frequency. Default is 600 (seconds), and the minimum value is 300.",
+    help="The scan interval in seconds for Modbus registers that are to be scanned at a low frequency. Default is 600 (seconds), and the minimum value is 1.",
 )
 _parser.add_argument(
     "--scan-interval-medium",
@@ -321,7 +355,7 @@ _parser.add_argument(
     dest=const.SIGENERGY2MQTT_SCAN_INTERVAL_MEDIUM,
     type=int,
     default=os.getenv(const.SIGENERGY2MQTT_SCAN_INTERVAL_MEDIUM, None),
-    help="The scan interval in seconds for Modbus registers that are to be scanned at a medium frequency. Default is 60 (seconds), and the minimum value is 30.",
+    help="The scan interval in seconds for Modbus registers that are to be scanned at a medium frequency. Default is 60 (seconds), and the minimum value is 1.",
 )
 _parser.add_argument(
     "--scan-interval-high",
@@ -330,7 +364,7 @@ _parser.add_argument(
     dest=const.SIGENERGY2MQTT_SCAN_INTERVAL_HIGH,
     type=int,
     default=os.getenv(const.SIGENERGY2MQTT_SCAN_INTERVAL_HIGH, None),
-    help="The scan interval in seconds for Modbus registers that are to be scanned at a high frequency. Default is 10 (seconds), and the minimum value is 5.",
+    help="The scan interval in seconds for Modbus registers that are to be scanned at a high frequency. Default is 10 (seconds), and the minimum value is 1.",
 )
 _parser.add_argument(
     "--scan-interval-realtime",
@@ -438,6 +472,27 @@ _parser.add_argument(
     const="true",
     help="Enable to send consumption data to PVOutput. If specified without a value, or with a value of 'true' or 'consumption', consumption data will be sent. With a value of 'imported', the energy imported from the grid will be sent as consumption. If not specified, no consumption data is sent.",
 )
+_parser.add_argument(
+    "--pvoutput-exports",
+    action="store_true",
+    dest=const.SIGENERGY2MQTT_PVOUTPUT_EXPORTS,
+    help="Enable to send export data to PVOutput.",
+)
+_parser.add_argument(
+    "--pvoutput-imports",
+    action="store_true",
+    dest=const.SIGENERGY2MQTT_PVOUTPUT_IMPORTS,
+    help="Enable to send import data to PVOutput.",
+)
+_parser.add_argument(
+    "--pvoutput-output-hour",
+    nargs="?",
+    action="store",
+    dest=const.SIGENERGY2MQTT_PVOUTPUT_OUTPUT_HOUR,
+    type=int,
+    default=os.getenv(const.SIGENERGY2MQTT_PVOUTPUT_OUTPUT_HOUR, None),
+    help="The hour of the day (20-23) at which the daily totals are sent to PVOutput. The default is 23 (11pm). Valid values are 20 to 23. The minute is randomly chosen between 51 and 58. If you specify -1, daily uploads will be sent at the same frequency as status updates.",
+)
 _parser.add_argument(
     "--pvoutput-interval",
     nargs="?",
@@ -455,6 +510,54 @@ _parser.add_argument(
     default=os.getenv(const.SIGENERGY2MQTT_PVOUTPUT_TEMP_TOPIC, None),
     help="An MQTT topic from which the current temperature can be read. This is used to send the temperature to PVOutput. If not specified, the temperature will not be sent to PVOutput.",
 )
+_parser.add_argument(
+    "--pvoutput-ext-v7",
+    nargs="?",
+    action="store",
+    dest=const.SIGENERGY2MQTT_PVOUTPUT_EXT_V7,
+    default=os.getenv(const.SIGENERGY2MQTT_PVOUTPUT_EXT_V7, None),
+    help="A sensor class name that will be used to populate the v7 extended field in PVOutput. If not specified, OR your donation status is not current, this field will not be sent to PVOutput. You can use any sensor with a numeric value.",
+)
+_parser.add_argument(
+    "--pvoutput-ext-v8",
+    nargs="?",
+    action="store",
+    dest=const.SIGENERGY2MQTT_PVOUTPUT_EXT_V8,
+    default=os.getenv(const.SIGENERGY2MQTT_PVOUTPUT_EXT_V8, None),
+    help="A sensor class name that will be used to populate the v8 extended field in PVOutput. If not specified, OR your donation status is not current, this field will not be sent to PVOutput. You can use any sensor with a numeric value.",
+)
+_parser.add_argument(
+    "--pvoutput-ext-v9",
+    nargs="?",
+    action="store",
+    dest=const.SIGENERGY2MQTT_PVOUTPUT_EXT_V9,
+    default=os.getenv(const.SIGENERGY2MQTT_PVOUTPUT_EXT_V9, None),
+    help="A sensor class name that will be used to populate the v9 extended field in PVOutput. If not specified, OR your donation status is not current, this field will not be sent to PVOutput. You can use any sensor with a numeric value.",
+)
+_parser.add_argument(
+    "--pvoutput-ext-v10",
+    nargs="?",
+    action="store",
+    dest=const.SIGENERGY2MQTT_PVOUTPUT_EXT_V10,
+    default=os.getenv(const.SIGENERGY2MQTT_PVOUTPUT_EXT_V10, None),
+    help="A sensor class name that will be used to populate the v10 extended field in PVOutput. If not specified, OR your donation status is not current, this field will not be sent to PVOutput. You can use any sensor with a numeric value.",
+)
+_parser.add_argument(
+    "--pvoutput-ext-v11",
+    nargs="?",
+    action="store",
+    dest=const.SIGENERGY2MQTT_PVOUTPUT_EXT_V11,
+    default=os.getenv(const.SIGENERGY2MQTT_PVOUTPUT_EXT_V11, None),
+    help="A sensor class name that will be used to populate the v11 extended field in PVOutput. If not specified, OR your donation status is not current, this field will not be sent to PVOutput. You can use any sensor with a numeric value.",
+)
+_parser.add_argument(
+    "--pvoutput-ext-v12",
+    nargs="?",
+    action="store",
+    dest=const.SIGENERGY2MQTT_PVOUTPUT_EXT_V12,
+    default=os.getenv(const.SIGENERGY2MQTT_PVOUTPUT_EXT_V12, None),
+    help="A sensor class name that will be used to populate the v12 extended field in PVOutput. If not specified, OR your donation status is not current, this field will not be sent to PVOutput. You can use any sensor with a numeric value.",
+)
 _parser.add_argument(
     "--pvoutput-log-level",
     action="store",
@@ -487,9 +590,10 @@ if _args.show_version:
 def apply_cli_to_env(variable: str, value: str) -> None:
     was = os.getenv(variable)
     if value is not None:
-        if value != was:
+        if str(value) != was:
             os.environ[variable] = str(value)
-            _logger.debug(f"Environment variable '{variable}' overridden from command line: set to '{value}' (was '{was}')")
+            if was is not None:
+                _logger.debug(f"Environment variable '{variable}' overridden from command line: set to '{'[REDACTED]' if 'PASSWORD' in variable or 'API_KEY' in variable else value}' (was '{was}')")
     else:
         if was is not None:
             os.environ[variable] = ""
@@ -523,8 +627,11 @@ for arg in vars(_args):
         or arg == const.SIGENERGY2MQTT_MQTT_TLS
         or arg == const.SIGENERGY2MQTT_MQTT_TLS_INSECURE
         or arg == const.SIGENERGY2MQTT_PVOUTPUT_ENABLED
+        or arg == const.SIGENERGY2MQTT_PVOUTPUT_EXPORTS
+        or arg == const.SIGENERGY2MQTT_PVOUTPUT_IMPORTS
         or arg == const.SIGENERGY2MQTT_SMARTPORT_ENABLED
         or arg == const.SIGENERGY2MQTT_NO_METRICS
+        or arg == const.SIGENERGY2MQTT_MODBUS_DISABLE_CHUNKING
     ) and getattr(_args, arg) not in ["true", "True", True, 1]:  # argparse will store false by default, so ignore unless actually specified (and therefore true)
         continue
     elif arg == const.SIGENERGY2MQTT_MODBUS_READ_ONLY and getattr(_args, arg) in ["true", "True", True, 1]:
