@@ -4,7 +4,7 @@ from .auto_discovery import scan as auto_discovery_scan
 from .home_assistant_config import HomeAssistantConfiguration
 from .modbus_config import DeviceConfig
 from .mqtt_config import MqttConfiguration
-from .pvoutput_config import PVOutputConfiguration
+from .pvoutput_config import CONSUMPTION, IMPORTED, PVOutputConfiguration
 from .validation import check_bool, check_host, check_float, check_int, check_int_list, check_log_level, check_port, check_string
 from pathlib import Path
 from ruamel.yaml import YAML
@@ -71,9 +71,10 @@ class Config:
             port = int(os.getenv(const.SIGENERGY2MQTT_MODBUS_PORT, "502"))
             logging.info(f"Auto-discovery required, scanning for Sigenergy devices ({port=})...")
             auto_discovered = auto_discovery_scan(port)
-            with open(auto_discovery_cache, "w") as f:
-                _yaml = YAML(typ="safe", pure=True)
-                _yaml.dump(auto_discovered, f)
+            if len(auto_discovered) > 0:
+                with open(auto_discovery_cache, "w") as f:
+                    _yaml = YAML(typ="safe", pure=True)
+                    _yaml.dump(auto_discovered, f)
         elif auto_discovery == "once" and auto_discovery_cache.is_file():
             logging.info("Auto-discovery already completed, using cached results.")
             with open(auto_discovery_cache, "r") as f:
@@ -81,7 +82,7 @@ class Config:
 
         for key, value in os.environ.items():
             if key.startswith("SIGENERGY2MQTT_") and key != "SIGENERGY2MQTT_CONFIG" and value is not None and value != "None":
-                logging.debug(f"Found env/cli override: {key} = {'******' if 'PASSWORD' in key or 'API_KEY' in key else value}")
+                logging.debug(f"Found env/cli override: {key} = {'[REDACTED]' if 'PASSWORD' in key or 'API_KEY' in key else value}")
                 try:
                     match key:
                         case const.SIGENERGY2MQTT_LOG_LEVEL:
@@ -107,8 +108,6 @@ class Config:
                             overrides["home-assistant"]["unique-id-prefix"] = check_string(os.environ[key], key)
                         case const.SIGENERGY2MQTT_HASS_USE_SIMPLIFIED_TOPICS:
                             overrides["home-assistant"]["use-simplified-topics"] = check_bool(os.environ[key], key)
-                        case const.SIGENERGY2MQTT_MODBUS_AUTO_DISCOVERY:
-                            check_string(os.environ[key], key, "force", "once", allow_none=True, allow_empty=True)
                         case const.SIGENERGY2MQTT_MODBUS_HOST:
                             overrides["modbus"][0]["host"] = check_host(os.environ[key], key)
                         case const.SIGENERGY2MQTT_MODBUS_PORT:
@@ -144,18 +143,24 @@ class Config:
                             if auto_discovered:
                                 for device in auto_discovered:
                                     device["write-only"] = overrides["modbus"][0]["write-only"]
+                        case const.SIGENERGY2MQTT_MODBUS_DISABLE_CHUNKING:
+                            overrides["modbus"][0]["disable-chunking"] = check_bool(os.environ[key], key)
+                        case const.SIGENERGY2MQTT_MODBUS_RETRIES:
+                            overrides["modbus"][0]["retries"] = check_int(os.environ[key], key, min=0)
+                        case const.SIGENERGY2MQTT_MODBUS_TIMEOUT:
+                            overrides["modbus"][0]["timeout"] = check_float(os.environ[key], key, min=0.25)
                         case const.SIGENERGY2MQTT_SCAN_INTERVAL_LOW:
-                            overrides["modbus"][0]["scan-interval-low"] = check_int(os.environ[key], key, min=300)
+                            overrides["modbus"][0]["scan-interval-low"] = check_int(os.environ[key], key, min=1)
                             if auto_discovered:
                                 for device in auto_discovered:
                                     device["scan-interval-low"] = overrides["modbus"][0]["scan-interval-low"]
                         case const.SIGENERGY2MQTT_SCAN_INTERVAL_MEDIUM:
-                            overrides["modbus"][0]["scan-interval-medium"] = check_int(os.environ[key], key, min=30)
+                            overrides["modbus"][0]["scan-interval-medium"] = check_int(os.environ[key], key, min=1)
                             if auto_discovered:
                                 for device in auto_discovered:
                                     device["scan-interval-medium"] = overrides["modbus"][0]["scan-interval-medium"]
                         case const.SIGENERGY2MQTT_SCAN_INTERVAL_HIGH:
-                            overrides["modbus"][0]["scan-interval-high"] = check_int(os.environ[key], key, min=5)
+                            overrides["modbus"][0]["scan-interval-high"] = check_int(os.environ[key], key, min=1)
                             if auto_discovered:
                                 for device in auto_discovered:
                                     device["scan-interval-high"] = overrides["modbus"][0]["scan-interval-high"]
@@ -184,6 +189,8 @@ class Config:
                             overrides["mqtt"]["broker"] = check_host(os.environ[key], key)
                         case const.SIGENERGY2MQTT_MQTT_PORT:
                             overrides["mqtt"]["port"] = check_port(os.environ[key], key)
+                        case const.SIGENERGY2MQTT_MQTT_KEEPALIVE:
+                            overrides["mqtt"]["keepalive"] = check_int(os.environ[key], key, min=1)
                         case const.SIGENERGY2MQTT_MQTT_TLS:
                             overrides["mqtt"]["tls"] = check_bool(os.environ[key], key)
                         case const.SIGENERGY2MQTT_MQTT_TLS_INSECURE:
@@ -198,18 +205,40 @@ class Config:
                             overrides["mqtt"]["password"] = os.environ[key]
                         case const.SIGENERGY2MQTT_PVOUTPUT_ENABLED:
                             overrides["pvoutput"]["enabled"] = check_bool(os.environ[key], key)
+                        case const.SIGENERGY2MQTT_PVOUTPUT_EXPORTS:
+                            overrides["pvoutput"]["exports"] = check_bool(os.environ[key], key)
+                        case const.SIGENERGY2MQTT_PVOUTPUT_IMPORTS:
+                            overrides["pvoutput"]["imports"] = check_bool(os.environ[key], key)
                         case const.SIGENERGY2MQTT_PVOUTPUT_LOG_LEVEL:
                             overrides["pvoutput"]["log-level"] = check_log_level(os.environ[key], key)
+                        case const.SIGENERGY2MQTT_PVOUTPUT_OUTPUT_HOUR:
+                            overrides["pvoutput"]["output-hour"] = check_int(os.environ[key], key, min=-1, max=23)
+                        case const.SIGENERGY2MQTT_PVOUTPUT_UPDATE_DEBUG_LOGGING:
+                            overrides["pvoutput"]["update-debug-logging"] = check_bool(os.environ[key], key)
                         case const.SIGENERGY2MQTT_PVOUTPUT_API_KEY:
                             overrides["pvoutput"]["api-key"] = check_string(os.environ[key], key, allow_none=False, allow_empty=False, hex_chars_only=True)
                         case const.SIGENERGY2MQTT_PVOUTPUT_SYSTEM_ID:
                             overrides["pvoutput"]["system-id"] = check_string(os.environ[key], key, allow_none=False, allow_empty=False)
                         case const.SIGENERGY2MQTT_PVOUTPUT_CONSUMPTION:
-                            overrides["pvoutput"]["consumption"] = check_string(os.environ[key], key, "false", "true", "consumption", "imported", allow_empty=False, allow_none=False)
+                            overrides["pvoutput"]["consumption"] = check_string(os.environ[key], key, "false", "true", CONSUMPTION, IMPORTED, allow_empty=False, allow_none=False)
                         case const.SIGENERGY2MQTT_PVOUTPUT_INTERVAL:
                             pass  # Deprecated
                         case const.SIGENERGY2MQTT_PVOUTPUT_TEMP_TOPIC:
                             overrides["pvoutput"]["temperature-topic"] = check_string(os.environ[key], key, allow_none=False, allow_empty=False)
+                        case const.SIGENERGY2MQTT_PVOUTPUT_EXT_V7:
+                            overrides["pvoutput"]["v7"] = check_string(os.environ[key], key, allow_none=True, allow_empty=True)
+                        case const.SIGENERGY2MQTT_PVOUTPUT_EXT_V8:
+                            overrides["pvoutput"]["v8"] = check_string(os.environ[key], key, allow_none=True, allow_empty=True)
+                        case const.SIGENERGY2MQTT_PVOUTPUT_EXT_V9:
+                            overrides["pvoutput"]["v9"] = check_string(os.environ[key], key, allow_none=True, allow_empty=True)
+                        case const.SIGENERGY2MQTT_PVOUTPUT_EXT_V10:
+                            overrides["pvoutput"]["v10"] = check_string(os.environ[key], key, allow_none=True, allow_empty=True)
+                        case const.SIGENERGY2MQTT_PVOUTPUT_EXT_V11:
+                            overrides["pvoutput"]["v11"] = check_string(os.environ[key], key, allow_none=True, allow_empty=True)
+                        case const.SIGENERGY2MQTT_PVOUTPUT_EXT_V12:
+                            overrides["pvoutput"]["v12"] = check_string(os.environ[key], key, allow_none=True, allow_empty=True)
+                        case const.SIGENERGY2MQTT_MODBUS_AUTO_DISCOVERY:
+                            pass  # Handled above
                         case _:
                             logging.warning(f"UNKNOWN env/cli override: {key} = {'******' if 'PASSWORD' in key or 'API_KEY' in key else value}")
                 except Exception as e:
@@ -240,8 +269,11 @@ class Config:
             else:
                 raise ValueError("Auto-discovery results must be a list of modbus device configurations")
 
-        if len(Config.devices) == 0:
-            raise ValueError("No Modbus devices configured")
+        if len(Config.devices) == 0 or (len(Config.devices) == 1 and not getattr(Config.devices[0], "host", None)):
+            if auto_discovery in ("once", "force"):
+                raise ValueError("No Modbus devices configured and auto-discovery did not find any Sigenergy devices; please check that the devices are powered on and reachable over the network")
+            else:
+                raise ValueError("No Modbus devices configured")
 
     @staticmethod
     def _configure(data: dict, override: bool = False) -> None:
