@@ -1,8 +1,8 @@
-__all__ = ["Device", "ModbusDevice", "ACCharger", "DCCharger", "ESS", "GridSensor", "Inverter", "PowerPlant", "PVString"]
+__all__ = ["Device", "ModbusDevice", "ACCharger", "DCCharger", "ESS", "GridSensor", "Inverter", "PowerPlant", "PVString", "DeviceRegistry"]
 
 from .ac_charger import ACCharger
 from .dc_charger import DCCharger
-from .device import Device, ModbusDevice
+from .device import Device, DeviceRegistry, ModbusDevice
 from .grid_sensor import GridSensor
 from .inverter import Inverter
 from .inverter_ess import ESS
