@@ -34,23 +34,23 @@ class PowerPlant(ModbusDevice):
         self._add_read_sensor(ro.MaxActivePower(plant_index))
         self._add_read_sensor(ro.MaxApparentPower(plant_index))
         self._add_read_sensor(ro.PlantBatterySoC(plant_index))
-        self._add_read_sensor(ro.PlantPhaseAActivePower(plant_index))
-        self._add_read_sensor(ro.PlantPhaseAReactivePower(plant_index))
+        self._add_read_sensor(ro.PlantPhaseActivePower(plant_index, "A"))
+        self._add_read_sensor(ro.PlantPhaseReactivePower(plant_index, "A"))
         if power_phases > 1:
-            self._add_read_sensor(ro.PlantPhaseBActivePower(plant_index))
-            self._add_read_sensor(ro.PlantPhaseBReactivePower(plant_index))
+            self._add_read_sensor(ro.PlantPhaseActivePower(plant_index, "B"))
+            self._add_read_sensor(ro.PlantPhaseReactivePower(plant_index, "B"))
         if power_phases > 2:
-            self._add_read_sensor(ro.PlantPhaseCActivePower(plant_index))
-            self._add_read_sensor(ro.PlantPhaseCReactivePower(plant_index))
+            self._add_read_sensor(ro.PlantPhaseActivePower(plant_index, "C"))
+            self._add_read_sensor(ro.PlantPhaseReactivePower(plant_index, "C"))
         self._add_read_sensor(ro.GeneralPCSAlarm(plant_index, ro.GeneralAlarm1(plant_index), ro.GeneralAlarm2(plant_index)))
         self._add_read_sensor(ro.GeneralAlarm3(plant_index))
         self._add_read_sensor(ro.GeneralAlarm4(plant_index))
         if len(Config.devices[plant_index].dc_chargers) > 0:
             self._add_read_sensor(ro.GeneralAlarm5(plant_index))
-        self._add_read_sensor(ro.PlantActivePower(plant_index))
-        self._add_read_sensor(ro.PlantReactivePower(plant_index))
-        self._add_read_sensor(plant_pv_power, "consumption")
-        self._add_read_sensor(battery_power, "consumption")
+        self._add_read_sensor(ro.PlantActivePower(plant_index), "Plant Power")
+        self._add_read_sensor(ro.PlantReactivePower(plant_index), "Plant Power")
+        self._add_read_sensor(plant_pv_power, "Consumption")
+        self._add_read_sensor(battery_power, "Consumption")
 
         self._add_derived_sensor(derived.BatteryChargingPower(plant_index, battery_power), battery_power)
         self._add_derived_sensor(derived.BatteryDischargingPower(plant_index, battery_power), battery_power)
@@ -70,32 +70,6 @@ class PowerPlant(ModbusDevice):
         self._add_read_sensor(ro.ChargeCutOffSoC(plant_index))
         self._add_read_sensor(ro.DischargeCutOffSoC(plant_index))
         self._add_read_sensor(ro.PlantBatterySoH(plant_index))
-
-        remote_ems = rw.RemoteEMS(plant_index)
-        self._add_read_sensor(remote_ems)
-        self._add_read_sensor(rw.RemoteEMSControlMode(plant_index, remote_ems))
-        self._add_read_sensor(rw.ActivePowerFixedAdjustmentTargetValue(plant_index, remote_ems))
-        self._add_read_sensor(rw.ReactivePowerFixedAdjustmentTargetValue(plant_index))
-        self._add_read_sensor(rw.ActivePowerPercentageAdjustmentTargetValue(plant_index, remote_ems))
-        self._add_read_sensor(rw.QSAdjustmentTargetValue(plant_index))
-        self._add_read_sensor(rw.PowerFactorAdjustmentTargetValue(plant_index))
-        self._add_read_sensor(rw.PhaseAActivePowerFixedAdjustmentTargetValue(plant_index, remote_ems, output_type))
-        self._add_read_sensor(rw.PhaseAReactivePowerFixedAdjustmentTargetValue(plant_index, remote_ems, output_type))
-        self._add_read_sensor(rw.PhaseAActivePowerPercentageAdjustmentTargetValue(plant_index, remote_ems, output_type))
-        self._add_read_sensor(rw.PhaseAQSAdjustmentTargetValue(plant_index, remote_ems, output_type))
-        if power_phases > 1:
-            self._add_read_sensor(rw.PhaseBActivePowerFixedAdjustmentTargetValue(plant_index, remote_ems, output_type))
-            self._add_read_sensor(rw.PhaseBReactivePowerFixedAdjustmentTargetValue(plant_index, remote_ems, output_type))
-            self._add_read_sensor(rw.PhaseBActivePowerPercentageAdjustmentTargetValue(plant_index, remote_ems, output_type))
-            self._add_read_sensor(rw.PhaseBQSAdjustmentTargetValue(plant_index, remote_ems, output_type))
-            self._add_read_sensor(rw.PhaseCActivePowerFixedAdjustmentTargetValue(plant_index, remote_ems, output_type))
-            self._add_read_sensor(rw.PhaseCReactivePowerFixedAdjustmentTargetValue(plant_index, remote_ems, output_type))
-            self._add_read_sensor(rw.PhaseCActivePowerPercentageAdjustmentTargetValue(plant_index, remote_ems, output_type))
-            self._add_read_sensor(rw.PhaseCQSAdjustmentTargetValue(plant_index, remote_ems, output_type))
-            self._add_read_sensor(rw.IndependentPhasePowerControl(plant_index, remote_ems, output_type))
-        self._add_read_sensor(rw.MaxChargingLimit(plant_index, remote_ems, rcp_value))
-        self._add_read_sensor(rw.MaxDischargingLimit(plant_index, remote_ems, rdp_value))
-        self._add_read_sensor(rw.PVMaxPowerLimit(plant_index, remote_ems))
         self._add_read_sensor(rw.GridMaxExportLimit(plant_index))
         self._add_read_sensor(rw.GridMaxImportLimit(plant_index))
         self._add_read_sensor(rw.PCSMaxExportLimit(plant_index))
@@ -106,6 +80,32 @@ class PowerPlant(ModbusDevice):
         self._add_read_sensor(ro.TotalLoadConsumption(plant_index))
         self._add_read_sensor(ro.TotalLoadDailyConsumption(plant_index))
 
+        remote_ems = rw.RemoteEMS(plant_index)
+        self._add_read_sensor(remote_ems)
+        self._add_read_sensor(rw.RemoteEMSControlMode(plant_index, remote_ems))
+        self._add_read_sensor(rw.ActivePowerFixedAdjustmentTargetValue(plant_index, remote_ems))
+        self._add_read_sensor(rw.ReactivePowerFixedAdjustmentTargetValue(plant_index))
+        self._add_read_sensor(rw.ActivePowerPercentageAdjustmentTargetValue(plant_index, remote_ems))
+        self._add_read_sensor(rw.QSAdjustmentTargetValue(plant_index))
+        self._add_read_sensor(rw.PowerFactorAdjustmentTargetValue(plant_index))
+        if output_type == 2:  # L1/L2/L3/N
+            self._add_read_sensor(rw.PhaseActivePowerFixedAdjustmentTargetValue(plant_index, remote_ems, output_type, "A"))
+            self._add_read_sensor(rw.PhaseReactivePowerFixedAdjustmentTargetValue(plant_index, remote_ems, output_type, "A"))
+            self._add_read_sensor(rw.PhaseActivePowerPercentageAdjustmentTargetValue(plant_index, remote_ems, output_type, "A"))
+            self._add_read_sensor(rw.PhaseQSAdjustmentTargetValue(plant_index, remote_ems, output_type, "A"))
+            self._add_read_sensor(rw.PhaseActivePowerFixedAdjustmentTargetValue(plant_index, remote_ems, output_type, "B"))
+            self._add_read_sensor(rw.PhaseReactivePowerFixedAdjustmentTargetValue(plant_index, remote_ems, output_type, "B"))
+            self._add_read_sensor(rw.PhaseActivePowerPercentageAdjustmentTargetValue(plant_index, remote_ems, output_type, "B"))
+            self._add_read_sensor(rw.PhaseQSAdjustmentTargetValue(plant_index, remote_ems, output_type, "B"))
+            self._add_read_sensor(rw.PhaseActivePowerFixedAdjustmentTargetValue(plant_index, remote_ems, output_type, "C"))
+            self._add_read_sensor(rw.PhaseReactivePowerFixedAdjustmentTargetValue(plant_index, remote_ems, output_type, "C"))
+            self._add_read_sensor(rw.PhaseActivePowerPercentageAdjustmentTargetValue(plant_index, remote_ems, output_type, "C"))
+            self._add_read_sensor(rw.PhaseQSAdjustmentTargetValue(plant_index, remote_ems, output_type, "C"))
+            self._add_read_sensor(rw.IndependentPhasePowerControl(plant_index, remote_ems, output_type))
+        self._add_read_sensor(rw.MaxChargingLimit(plant_index, remote_ems, rcp_value))
+        self._add_read_sensor(rw.MaxDischargingLimit(plant_index, remote_ems, rdp_value))
+        self._add_read_sensor(rw.PVMaxPowerLimit(plant_index, remote_ems))
+
         address = 30098
         for n in range(1, 25):
             self._add_read_sensor(ro.SmartLoadTotalConsumption(plant_index, address, n))
@@ -154,7 +154,7 @@ class PowerPlant(ModbusDevice):
             self._add_read_sensor(plant_3rd_party_pv_power)
             total_pv_power.register_source_sensors(plant_3rd_party_pv_power, type=derived.TotalPVPower.SourceType.FAILOVER, enabled=False)
         else:
-            self._add_read_sensor(plant_3rd_party_pv_power, "consumption")
+            self._add_read_sensor(plant_3rd_party_pv_power, "Consumption")
             total_pv_power.register_source_sensors(plant_3rd_party_pv_power, type=derived.TotalPVPower.SourceType.MANDATORY, enabled=True)
 
         self._add_derived_sensor(total_pv_power, plant_3rd_party_pv_power)
@@ -165,8 +165,8 @@ class PowerPlant(ModbusDevice):
         plant_lifetime_pv_energy = ro.PlantPVTotalGeneration(plant_index)
         plant_3rd_party_lifetime_pv_energy = ro.ThirdPartyLifetimePVEnergy(plant_index)
         total_lifetime_pv_energy = derived.TotalLifetimePVEnergy(plant_index)
-        self._add_read_sensor(plant_lifetime_pv_energy, "lifetime_production")
-        self._add_read_sensor(plant_3rd_party_lifetime_pv_energy, "lifetime_production")
+        self._add_read_sensor(plant_lifetime_pv_energy, "Lifetime Production")
+        self._add_read_sensor(plant_3rd_party_lifetime_pv_energy, "Lifetime Production")
         self._add_derived_sensor(total_lifetime_pv_energy, plant_lifetime_pv_energy, plant_3rd_party_lifetime_pv_energy)
         self._add_derived_sensor(derived.PlantDailyPVEnergy(plant_index, plant_lifetime_pv_energy), plant_lifetime_pv_energy)
         self._add_derived_sensor(derived.TotalDailyPVEnergy(plant_index, total_lifetime_pv_energy), total_lifetime_pv_energy)
