@@ -2,12 +2,11 @@ from .const import PERCENTAGE, DeviceClass, InputType, StateClass, UnitOfEnergy
 from .sanity_check import SanityCheck
 from concurrent.futures import Future
 from pathlib import Path
-from pymodbus.client import AsyncModbusTcpClient as ModbusClient
 from pymodbus.pdu import ExceptionResponse
 from sigenergy2mqtt.config import Config, RegisterAccess
 from sigenergy2mqtt.devices.types import HybridInverter, PVInverter
 from sigenergy2mqtt.metrics.metrics import Metrics
-from sigenergy2mqtt.modbus import ModbusLockFactory
+from sigenergy2mqtt.modbus import ModbusClient, ModbusLockFactory
 from sigenergy2mqtt.mqtt import MqttClient, MqttHandler
 from typing import Any, Coroutine, Dict, Final
 import abc
@@ -232,7 +231,7 @@ class Sensor(Dict[str, any], metaclass=abc.ABCMeta):
 
     def apply_sensor_overrides(self, registers: RegisterAccess):
         for identifier in Config.sensor_overrides.keys():
-            if identifier in self.__class__.__name__ or identifier in self["object_id"]:
+            if identifier in self.__class__.__name__ or identifier in self["object_id"] or identifier in self.unique_id:
                 overrides = Config.sensor_overrides[identifier]
                 if "debug-logging" in overrides and self._debug_logging != overrides["debug-logging"]:
                     self._debug_logging = overrides["debug-logging"]
@@ -325,6 +324,9 @@ class Sensor(Dict[str, any], metaclass=abc.ABCMeta):
         if self._debug_logging:
             logging.debug(f"{self.__class__.__name__} Getting discovery")
         components = self.get_discovery_components()
+        for config in components.values():
+            if "object_id" in config:
+                config["default_entity_id"] = f"{config['platform']}.{config['object_id']}"
         if self.publishable and not Config.clean:
             if self._persistent_publish_state_file.exists():
                 self._persistent_publish_state_file.unlink(missing_ok=True)
@@ -348,7 +350,10 @@ class Sensor(Dict[str, any], metaclass=abc.ABCMeta):
         return components
 
     def get_discovery_components(self) -> Dict[str, Dict[str, Any]]:
-        return {self.unique_id: dict(dict((k, v) for k, v in self.items() if v is not None))}
+        components = dict((k, v) for k, v in self.items() if v is not None)
+        if "options" in components and self["platform"] != "select":
+            del components["options"]
+        return {self.unique_id: dict(components)}
 
     async def get_state(self, raw: bool = False, republish: bool = False, **kwargs) -> float | int | str | None:
         """Gets the state of this sensor.
@@ -392,7 +397,7 @@ class Sensor(Dict[str, any], metaclass=abc.ABCMeta):
                             logging.debug(f"{self.__class__.__name__} Publishing SKIPPED: State is None?")
                     else:
                         if self._failures > 0:
-                            logging.info(f"{self.__class__.__name__} Resetting failure count from {self._failures} to 0")
+                            logging.info(f"{self.__class__.__name__} Resetting failure count from {self._failures} to 0 because valid state acquired ({state=})")
                             self._failures = 0
                             self._next_retry = None
                         if self.debug_logging:
@@ -469,6 +474,31 @@ class Sensor(Dict[str, any], metaclass=abc.ABCMeta):
             if len(self._states) > self._max_states:
                 self._states = self._states[-self._max_states :]
 
+    def state2raw(self, state: float | int | str) -> float | int | str:
+        """Converts a processed state back to its raw value.
+
+        Args:
+            state:      The processed state.
+
+        Returns:
+            The raw state.
+        """
+        if state is None:
+            return None
+        elif isinstance(state, str):
+            if self._data_type == ModbusClient.DATATYPE.STRING:
+                return state
+            elif "options" in self and state in self["options"]:
+                return self["options"].index(state)
+        try:
+            value = float(state) if "." in state else int(state)
+        except ValueError:
+            value = state
+        if isinstance(value, (float, int)):
+            if self.gain is not None and self.gain != 1:
+                value *= self.gain
+        return int(value)
+
 
 class RequisiteSensor(Sensor):
     """Base superclass of all sensor definitions that are required by other sensors.
@@ -723,17 +753,16 @@ class ReadOnlySensor(ModbusSensor, ReadableSensorMixin):
             )
 
         try:
-            async with ModbusLockFactory.get(modbus).lock():
-                start = time.monotonic()
-                if self._input_type == InputType.HOLDING:
-                    rr = await modbus.read_holding_registers(self._address, count=self._count, device_id=self._device_address)
-                elif self._input_type == InputType.INPUT:
-                    rr = await modbus.read_input_registers(self._address, count=self._count, device_id=self._device_address)
-                else:
-                    logging.error(f"{self.__class__.__name__} Unknown input type '{self._input_type}'")
-                    raise Exception(f"Unknown input type '{self._input_type}'")
-                elapsed = time.monotonic() - start
-                await Metrics.modbus_read(self._count, elapsed)
+            start = time.monotonic()
+            if self._input_type == InputType.HOLDING:
+                rr = await modbus.read_holding_registers(self._address, count=self._count, device_id=self._device_address, trace=self.debug_logging)
+            elif self._input_type == InputType.INPUT:
+                rr = await modbus.read_input_registers(self._address, count=self._count, device_id=self._device_address, trace=self.debug_logging)
+            else:
+                logging.error(f"{self.__class__.__name__} Unknown input type '{self._input_type}'")
+                raise Exception(f"Unknown input type '{self._input_type}'")
+            elapsed = time.monotonic() - start
+            await Metrics.modbus_read(self._count, elapsed)
             result = self._check_register_response(rr, f"read_{self._input_type}_registers")
             if result:
                 self.set_latest_state(modbus.convert_from_registers(rr.registers, self._data_type))
@@ -747,6 +776,10 @@ class ReadOnlySensor(ModbusSensor, ReadableSensorMixin):
             await Metrics.modbus_read_error()
             raise
 
+        if self.debug_logging:
+            logging.debug(
+                f"{self.__class__.__name__} read_{self._input_type}_registers({self._address}, count={self._count}, device_id={self._device_address}) plant_index={self._plant_index} interval={self._scan_interval}s actual={None if len(self._states) == 0 else str(round(time.time() - self._states[-1][0], 2)) + 's'} elapsed={(elapsed / 1000):.2f}ms {result=}"
+            )
         return result
 
     def get_attributes(self) -> dict[str, Any]:
@@ -865,6 +898,9 @@ class TimestampSensor(ReadOnlySensor):
             dt_object = datetime.datetime.fromtimestamp(value, datetime.timezone.utc)
             return dt_object.isoformat()
 
+    def state2raw(self, state) -> float | int | str:
+        return int(datetime.datetime.fromisoformat(state).timestamp())
+
 
 class ObservableMixin(abc.ABC):
     @abc.abstractmethod
@@ -953,6 +989,8 @@ class WriteOnlySensor(WritableSensorMixin):
         name_on: str = "Power On",
         icon_off: str = "mdi:power-off",
         icon_on: str = "mdi:power-on",
+        value_off: int = 0,
+        value_on: int = 1,
     ):
         super().__init__(
             name,
@@ -977,6 +1015,7 @@ class WriteOnlySensor(WritableSensorMixin):
         self._payloads = {"off": payload_off, "on": payload_on}
         self._names = {"off": name_off, "on": name_on}
         self._icons = {"off": icon_off, "on": icon_on}
+        self._values = {"off": value_off, "on": value_on}
 
     def get_discovery_components(self) -> Dict[str, dict[str, Any]]:
         components = {}
@@ -1001,9 +1040,9 @@ class WriteOnlySensor(WritableSensorMixin):
 
     async def set_value(self, modbus: ModbusClient, mqtt: MqttClient, value: float | int | str, source: str, handler: MqttHandler) -> bool:
         if value == self._payloads["off"]:
-            return await super().set_value(modbus, mqtt, 0, source, handler)
+            return await super().set_value(modbus, mqtt, self._values["off"], source, handler)
         elif value == self._payloads["on"]:
-            return await super().set_value(modbus, mqtt, 1, source, handler)
+            return await super().set_value(modbus, mqtt, self._values["on"], source, handler)
         else:
             logging.warning(f"{self.__class__.__name__} Ignored attempt to set value to {value}: Must be either '{self._payloads['on']}' or '{self._payloads['off']}'")
         return False
@@ -1489,9 +1528,21 @@ class AlarmCombinedSensor(Sensor, ReadableSensorMixin, HybridInverter, PVInverte
             gain=None,
             precision=None,
         )
+        device_addresses = set([a._device_address for a in alarms])
+        first_address = min([a._address for a in alarms])
+        last_address = max([a._address + a._count - 1 for a in alarms])
+        count = sum([a._count for a in alarms])
+        assert len(device_addresses) == 1, f"{self.__class__.__name__} Combined alarms must have the same device address ({device_addresses})"
+        assert (last_address - first_address + 1) == count, f"{self.__class__.__name__} Combined alarms must have contiguous address ranges ({[a._address for a in alarms]})"
         ReadableSensorMixin.__init__(self, scan_interval=10)
         self["enabled_by_default"] = True
         self._alarms = list(alarms)
+        self._address = min([a._address for a in alarms])
+        self._device_address = device_addresses.pop()
+        self._scan_interval = min([a._scan_interval for a in alarms])
+        self._count = count
+        self._input_type = alarms[0]._input_type
+        self._data_type = alarms[0]._data_type
 
     async def get_state(self, raw: bool = False, republish: bool = False, **kwargs) -> float | int | str | None:
         """Gets the state of this sensor.
@@ -1552,6 +1603,16 @@ class RunningStateSensor(ReadOnlySensor):
             precision=None,
         )
         self["enabled_by_default"] = True
+        self["options"] = [
+            "Standby",  # 0
+            "Normal",  # 1
+            "Fault",  # 2
+            "Power-Off",  # 3
+            None,  # 4
+            None,  # 5
+            None,  # 6
+            "Environmental Abnormality",  # 7
+        ]
 
     async def get_state(self, raw: bool = False, republish: bool = False, **kwargs) -> float | int | str | None:
         """Gets the state of this sensor.
@@ -1569,16 +1630,8 @@ class RunningStateSensor(ReadOnlySensor):
             return value
         elif value is None:
             return None
-        elif value == 0:
-            return "Standby"
-        elif value == 1:
-            return "Normal"
-        elif value == 2:
-            return "Fault"
-        elif value == 3:
-            return "Power-Off"
-        elif value == 7:
-            return "Environmental Abnormality"
+        elif 0 <= value <= (len(self["options"]) - 1) and self["options"][value] is not None:
+            return self["options"][value]
         else:
             return f"Unknown State code: {value}"
 
@@ -1662,7 +1715,7 @@ class EnergyLifetimeAccumulationSensor(ResettableAccumulationSensor):
                     content = f.read()
                     if content is not None and content != "None":
                         self._current_total = float(content)
-                        logging.info(f"{self.__class__.__name__} Loaded current state from {self._persistent_state_file} ({self._current_total})")
+                        logging.debug(f"{self.__class__.__name__} Loaded current state from {self._persistent_state_file} ({self._current_total})")
                 except ValueError as error:
                     logging.warning(f"{self.__class__.__name__} Failed to read {self._persistent_state_file}: {error}")
         else:
@@ -1801,13 +1854,11 @@ class EnergyDailyAccumulationSensor(ResettableAccumulationSensor):
         self.set_latest_state(self._state_now)
 
     async def _update_state_at_midnight(self, midnight_state: float) -> None:
-        if midnight_state:
+        if midnight_state is not None:
             async with self._state_at_midnight_lock:
                 with self._persistent_state_file.open("w") as f:
                     f.write(str(midnight_state))
                 self._state_at_midnight = midnight_state
-        else:
-            logging.warning(f"{self.__class__.__name__} Ignored attempt to update midnight state to {midnight_state}")
 
 
 class PVPowerSensor:
