@@ -161,16 +161,25 @@ class PowerFactorAdjustmentTargetValue(NumericSensor, HybridInverter, PVInverter
         return attributes
 
 
-class PhaseAActivePowerFixedAdjustmentTargetValue(NumericSensor, HybridInverter):
-    def __init__(self, plant_index: int, remote_ems: RemoteEMSMixin, output_type: int):
+class PhaseActivePowerFixedAdjustmentTargetValue(NumericSensor, HybridInverter):
+    def __init__(self, plant_index: int, remote_ems: RemoteEMSMixin, output_type: int, phase: str):
+        match phase:
+            case "A":
+                address = 40008
+            case "B":
+                address = 40010
+            case "C":
+                address = 40012
+            case _:
+                raise ValueError("Phase must be 'A', 'B', or 'C'")
         super().__init__(
             remote_ems=remote_ems,
-            name="Phase A Active Power Fixed Adjustment Target Value",
-            object_id=f"{Config.home_assistant.entity_id_prefix}_{plant_index}_plant_phase_a_active_power_fixed_adjustment_target_value",
+            name=f"Phase {phase} Active Power Fixed Adjustment Target Value",
+            object_id=f"{Config.home_assistant.entity_id_prefix}_{plant_index}_plant_phase_{phase.lower()}_active_power_fixed_adjustment_target_value",
             input_type=InputType.HOLDING,
             plant_index=plant_index,
             device_address=247,
-            address=40008,
+            address=address,
             count=2,
             data_type=ModbusClient.DATATYPE.INT32,
             scan_interval=Config.devices[plant_index].scan_interval.medium if plant_index < len(Config.devices) else 60,
@@ -190,74 +199,25 @@ class PhaseAActivePowerFixedAdjustmentTargetValue(NumericSensor, HybridInverter)
         return attributes
 
 
-class PhaseBActivePowerFixedAdjustmentTargetValue(NumericSensor, HybridInverter):
-    def __init__(self, plant_index: int, remote_ems: RemoteEMSMixin, output_type: int):
+class PhaseReactivePowerFixedAdjustmentTargetValue(NumericSensor, HybridInverter):
+    def __init__(self, plant_index: int, remote_ems: RemoteEMSMixin, output_type: int, phase: str):
+        match phase:
+            case "A":
+                address = 40014
+            case "B":
+                address = 40016
+            case "C":
+                address = 40018
+            case _:
+                raise ValueError("Phase must be 'A', 'B', or 'C'")
         super().__init__(
             remote_ems=remote_ems,
-            name="Phase B Active Power Fixed Adjustment Target Value",
-            object_id=f"{Config.home_assistant.entity_id_prefix}_{plant_index}_plant_phase_b_active_power_fixed_adjustment_target_value",
+            name=f"Phase {phase} Reactive Power Fixed Adjustment Target Value",
+            object_id=f"{Config.home_assistant.entity_id_prefix}_{plant_index}_plant_phase_{phase.lower()}_reactive_power_fixed_adjustment_target_value",
             input_type=InputType.HOLDING,
             plant_index=plant_index,
             device_address=247,
-            address=40010,
-            count=2,
-            data_type=ModbusClient.DATATYPE.INT32,
-            scan_interval=Config.devices[plant_index].scan_interval.medium if plant_index < len(Config.devices) else 60,
-            unit=UnitOfPower.KILO_WATT,
-            device_class=DeviceClass.POWER,
-            state_class=None,
-            icon="mdi:lightning-bolt",
-            gain=1000,
-            precision=2,
-        )
-        if output_type != 2:  # L1/L2/L3/N
-            self.publishable = False
-
-    def get_attributes(self) -> dict[str, Any]:
-        attributes = super().get_attributes()
-        attributes["comment"] = "Valid only when Output Type is L1/L2/L3/N"
-        return attributes
-
-
-class PhaseCActivePowerFixedAdjustmentTargetValue(NumericSensor, HybridInverter):
-    def __init__(self, plant_index: int, remote_ems: RemoteEMSMixin, output_type: int):
-        super().__init__(
-            remote_ems=remote_ems,
-            name="Phase C Active Power Fixed Adjustment Target Value",
-            object_id=f"{Config.home_assistant.entity_id_prefix}_{plant_index}_plant_phase_c_active_power_fixed_adjustment_target_value",
-            input_type=InputType.HOLDING,
-            plant_index=plant_index,
-            device_address=247,
-            address=40012,
-            count=2,
-            data_type=ModbusClient.DATATYPE.INT32,
-            scan_interval=Config.devices[plant_index].scan_interval.medium if plant_index < len(Config.devices) else 60,
-            unit=UnitOfPower.KILO_WATT,
-            device_class=DeviceClass.POWER,
-            state_class=None,
-            icon="mdi:lightning-bolt",
-            gain=1000,
-            precision=2,
-        )
-        if output_type != 2:  # L1/L2/L3/N
-            self.publishable = False
-
-    def get_attributes(self) -> dict[str, Any]:
-        attributes = super().get_attributes()
-        attributes["comment"] = "Valid only when Output Type is L1/L2/L3/N"
-        return attributes
-
-
-class PhaseAReactivePowerFixedAdjustmentTargetValue(NumericSensor, HybridInverter):
-    def __init__(self, plant_index: int, remote_ems: RemoteEMSMixin, output_type: int):
-        super().__init__(
-            remote_ems=remote_ems,
-            name="Phase A Reactive Power Fixed Adjustment Target Value",
-            object_id=f"{Config.home_assistant.entity_id_prefix}_{plant_index}_plant_phase_a_reactive_power_fixed_adjustment_target_value",
-            input_type=InputType.HOLDING,
-            plant_index=plant_index,
-            device_address=247,
-            address=40014,
+            address=address,
             count=2,
             data_type=ModbusClient.DATATYPE.INT32,
             scan_interval=Config.devices[plant_index].scan_interval.medium if plant_index < len(Config.devices) else 60,
@@ -277,74 +237,25 @@ class PhaseAReactivePowerFixedAdjustmentTargetValue(NumericSensor, HybridInverte
         return attributes
 
 
-class PhaseBReactivePowerFixedAdjustmentTargetValue(NumericSensor, HybridInverter):
-    def __init__(self, plant_index: int, remote_ems: RemoteEMSMixin, output_type: int):
+class PhaseActivePowerPercentageAdjustmentTargetValue(NumericSensor, HybridInverter):
+    def __init__(self, plant_index: int, remote_ems: RemoteEMSMixin, output_type: int, phase: str):
+        match phase:
+            case "A":
+                address = 40020
+            case "B":
+                address = 40021
+            case "C":
+                address = 40022
+            case _:
+                raise ValueError("Phase must be 'A', 'B', or 'C'")
         super().__init__(
             remote_ems=remote_ems,
-            name="Phase B Reactive Power Fixed Adjustment Target Value",
-            object_id=f"{Config.home_assistant.entity_id_prefix}_{plant_index}_plant_phase_b_reactive_power_fixed_adjustment_target_value",
+            name=f"Phase {phase} Active Power Percentage Adjustment Target Value",
+            object_id=f"{Config.home_assistant.entity_id_prefix}_{plant_index}_plant_phase_{phase.lower()}_active_power_percentage_adjustment_target_value",
             input_type=InputType.HOLDING,
             plant_index=plant_index,
             device_address=247,
-            address=40016,
-            count=2,
-            data_type=ModbusClient.DATATYPE.INT32,
-            scan_interval=Config.devices[plant_index].scan_interval.medium if plant_index < len(Config.devices) else 60,
-            unit=UnitOfReactivePower.KILO_VOLT_AMPERE_REACTIVE,
-            device_class=None,
-            state_class=None,
-            icon="mdi:lightning-bolt",
-            gain=1000,
-            precision=2,
-        )
-        if output_type != 2:  # L1/L2/L3/N
-            self.publishable = False
-
-    def get_attributes(self) -> dict[str, Any]:
-        attributes = super().get_attributes()
-        attributes["comment"] = "Valid only when Output Type is L1/L2/L3/N"
-        return attributes
-
-
-class PhaseCReactivePowerFixedAdjustmentTargetValue(NumericSensor, HybridInverter):
-    def __init__(self, plant_index: int, remote_ems: RemoteEMSMixin, output_type: int):
-        super().__init__(
-            remote_ems=remote_ems,
-            name="Phase C Reactive Power Fixed Adjustment Target Value",
-            object_id=f"{Config.home_assistant.entity_id_prefix}_{plant_index}_plant_phase_c_reactive_power_fixed_adjustment_target_value",
-            input_type=InputType.HOLDING,
-            plant_index=plant_index,
-            device_address=247,
-            address=40018,
-            count=2,
-            data_type=ModbusClient.DATATYPE.INT32,
-            scan_interval=Config.devices[plant_index].scan_interval.medium if plant_index < len(Config.devices) else 60,
-            unit=UnitOfReactivePower.KILO_VOLT_AMPERE_REACTIVE,
-            device_class=None,
-            state_class=None,
-            icon="mdi:lightning-bolt",
-            gain=1000,
-            precision=2,
-        )
-        if output_type != 2:  # L1/L2/L3/N
-            self.publishable = False
-
-    def get_attributes(self) -> dict[str, Any]:
-        attributes = super().get_attributes()
-        attributes["comment"] = "Valid only when Output Type is L1/L2/L3/N"
-        return attributes
-
-
-class PhaseAActivePowerPercentageAdjustmentTargetValue(NumericSensor, HybridInverter):
-    def __init__(self, plant_index: int, remote_ems: RemoteEMSMixin, output_type: int):
-        super().__init__(
-            remote_ems=remote_ems,
-            name="Phase A Active Power Percentage Adjustment Target Value",
-            object_id=f"{Config.home_assistant.entity_id_prefix}_{plant_index}_plant_phase_a_active_power_percentage_adjustment_target_value",
-            input_type=InputType.HOLDING,
-            plant_index=plant_index,
-            device_address=247,
-            address=40020,
+            address=address,
             count=1,
             data_type=ModbusClient.DATATYPE.INT16,
             scan_interval=Config.devices[plant_index].scan_interval.medium if plant_index < len(Config.devices) else 60,
@@ -366,140 +277,25 @@ class PhaseAActivePowerPercentageAdjustmentTargetValue(NumericSensor, HybridInve
         return attributes
 
 
-class PhaseBActivePowerPercentageAdjustmentTargetValue(NumericSensor, HybridInverter):
-    def __init__(self, plant_index: int, remote_ems: RemoteEMSMixin, output_type: int):
+class PhaseQSAdjustmentTargetValue(NumericSensor, HybridInverter):
+    def __init__(self, plant_index: int, remote_ems: RemoteEMSMixin, output_type: int, phase: str):
+        match phase:
+            case "A":
+                address = 40023
+            case "B":
+                address = 40024
+            case "C":
+                address = 40025
+            case _:
+                raise ValueError("Phase must be 'A', 'B', or 'C'")
         super().__init__(
             remote_ems=remote_ems,
-            name="Phase B Active Power Percentage Adjustment Target Value",
-            object_id=f"{Config.home_assistant.entity_id_prefix}_{plant_index}_plant_phase_b_active_power_percentage_adjustment_target_value",
+            name=f"Phase {phase} Q/S Fixed Adjustment Target Value",
+            object_id=f"{Config.home_assistant.entity_id_prefix}_{plant_index}_plant_phase_{phase.lower()}_q_s_fixed_adjustment_target_value",
             input_type=InputType.HOLDING,
             plant_index=plant_index,
             device_address=247,
-            address=40021,
-            count=1,
-            data_type=ModbusClient.DATATYPE.INT16,
-            scan_interval=Config.devices[plant_index].scan_interval.medium if plant_index < len(Config.devices) else 60,
-            unit=PERCENTAGE,
-            device_class=None,
-            state_class=None,
-            icon="mdi:percent",
-            gain=100,
-            precision=None,
-            min=-100.00,
-            max=100.00,
-        )
-        if output_type != 2:  # L1/L2/L3/N
-            self.publishable = False
-
-    def get_attributes(self) -> dict[str, Any]:
-        attributes = super().get_attributes()
-        attributes["comment"] = "Valid only when Output Type is L1/L2/L3/N. Range: [-100.00,100.00]"
-        return attributes
-
-
-class PhaseCActivePowerPercentageAdjustmentTargetValue(NumericSensor, HybridInverter):
-    def __init__(self, plant_index: int, remote_ems: RemoteEMSMixin, output_type: int):
-        super().__init__(
-            remote_ems=remote_ems,
-            name="Phase C Active Power Percentage Adjustment Target Value",
-            object_id=f"{Config.home_assistant.entity_id_prefix}_{plant_index}_plant_phase_c_active_power_percentage_adjustment_target_value",
-            input_type=InputType.HOLDING,
-            plant_index=plant_index,
-            device_address=247,
-            address=40022,
-            count=1,
-            data_type=ModbusClient.DATATYPE.INT16,
-            scan_interval=Config.devices[plant_index].scan_interval.medium if plant_index < len(Config.devices) else 60,
-            unit=PERCENTAGE,
-            device_class=None,
-            state_class=None,
-            icon="mdi:percent",
-            gain=100,
-            precision=None,
-            min=-100.00,
-            max=100.00,
-        )
-        if output_type != 2:  # L1/L2/L3/N
-            self.publishable = False
-
-    def get_attributes(self) -> dict[str, Any]:
-        attributes = super().get_attributes()
-        attributes["comment"] = "Valid only when Output Type is L1/L2/L3/N. Range: [-100.00,100.00]"
-        return attributes
-
-
-class PhaseAQSAdjustmentTargetValue(NumericSensor, HybridInverter):
-    def __init__(self, plant_index: int, remote_ems: RemoteEMSMixin, output_type: int):
-        super().__init__(
-            remote_ems=remote_ems,
-            name="Phase A Q/S Fixed Adjustment Target Value",
-            object_id=f"{Config.home_assistant.entity_id_prefix}_{plant_index}_plant_phase_a_q_s_fixed_adjustment_target_value",
-            input_type=InputType.HOLDING,
-            plant_index=plant_index,
-            device_address=247,
-            address=40023,
-            count=1,
-            data_type=ModbusClient.DATATYPE.INT16,
-            scan_interval=Config.devices[plant_index].scan_interval.medium if plant_index < len(Config.devices) else 60,
-            unit=PERCENTAGE,
-            device_class=None,
-            state_class=None,
-            icon="mdi:percent",
-            gain=100,
-            precision=None,
-            min=-60.00,
-            max=60.00,
-        )
-        if output_type != 2:  # L1/L2/L3/N
-            self.publishable = False
-
-    def get_attributes(self) -> dict[str, Any]:
-        attributes = super().get_attributes()
-        attributes["comment"] = "Valid only when Output Type is L1/L2/L3/N. Range: [-60.00,60.00]"
-        return attributes
-
-
-class PhaseBQSAdjustmentTargetValue(NumericSensor, HybridInverter):
-    def __init__(self, plant_index: int, remote_ems: RemoteEMSMixin, output_type: int):
-        super().__init__(
-            remote_ems=remote_ems,
-            name="Phase B Q/S Fixed Adjustment Target Value",
-            object_id=f"{Config.home_assistant.entity_id_prefix}_{plant_index}_plant_phase_b_q_s_fixed_adjustment_target_value",
-            input_type=InputType.HOLDING,
-            plant_index=plant_index,
-            device_address=247,
-            address=40024,
-            count=1,
-            data_type=ModbusClient.DATATYPE.INT16,
-            scan_interval=Config.devices[plant_index].scan_interval.medium if plant_index < len(Config.devices) else 60,
-            unit=PERCENTAGE,
-            device_class=None,
-            state_class=None,
-            icon="mdi:percent",
-            gain=100,
-            precision=None,
-            min=-60.00,
-            max=60.00,
-        )
-        if output_type != 2:  # L1/L2/L3/N
-            self.publishable = False
-
-    def get_attributes(self) -> dict[str, Any]:
-        attributes = super().get_attributes()
-        attributes["comment"] = "Valid only when Output Type is L1/L2/L3/N. Range: [-60.00,60.00]"
-        return attributes
-
-
-class PhaseCQSAdjustmentTargetValue(NumericSensor, HybridInverter):
-    def __init__(self, plant_index: int, remote_ems: RemoteEMSMixin, output_type: int):
-        super().__init__(
-            remote_ems=remote_ems,
-            name="Phase C Q/S Fixed Adjustment Target Value",
-            object_id=f"{Config.home_assistant.entity_id_prefix}_{plant_index}_plant_phase_c_q_s_fixed_adjustment_target_value",
-            input_type=InputType.HOLDING,
-            plant_index=plant_index,
-            device_address=247,
-            address=40025,
+            address=address,
             count=1,
             data_type=ModbusClient.DATATYPE.INT16,
             scan_interval=Config.devices[plant_index].scan_interval.medium if plant_index < len(Config.devices) else 60,
@@ -615,41 +411,25 @@ class RemoteEMSControlMode(ReadWriteSensor, HybridInverter, PVInverter):
             return value
         elif value is None:
             return None
-        elif value == 0:
-            return "PCS remote control"
-        elif value == 1:
-            return "Standby"
-        elif value == 2:
-            return "Maximum Self-consumption (Default)"
-        elif value == 3:
-            return "Command Charging (Consume power from the grid first)"
-        elif value == 4:
-            return "Command Charging (Consume power from the PV first)"
-        elif value == 5:
-            return "Command Discharging (Output power from PV first)"
-        elif value == 6:
-            return "Command Discharging (Output power from the battery first)"
+        elif 0 <= value <= (len(self["options"]) - 1):
+            return self["options"][value]
         else:
             return f"Unknown Mode: {value}"
 
     async def set_value(self, modbus: ModbusClient, mqtt: MqttClient, value: float | int | str, source: str, handler: MqttHandler) -> bool | Exception | ExceptionResponse:
         result = False
-        if value == "PCS remote control":
-            result = await super().set_value(modbus, mqtt, 0, source, handler)
-        elif value == "Standby":
-            result = await super().set_value(modbus, mqtt, 1, source, handler)
-        elif value == "Maximum Self-consumption (Default)":
-            result = await super().set_value(modbus, mqtt, 2, source, handler)
-        elif value == "Command Charging (Consume power from the grid first)":
-            result = await super().set_value(modbus, mqtt, 3, source, handler)
-        elif value == "Command Charging (Consume power from the PV first)":
-            result = await super().set_value(modbus, mqtt, 4, source, handler)
-        elif value == "Command Discharging (Output power from PV first)":
-            result = await super().set_value(modbus, mqtt, 5, source, handler)
-        elif value == "Command Discharging (Output power from the battery first)":
-            result = await super().set_value(modbus, mqtt, 6, source, handler)
+        index = None
+        try:
+            index = self["options"].index(value)
+        except ValueError:
+            try:
+                index = int(value)
+            except ValueError:
+                pass
+        if index is not None and 0 <= index <= 6:
+            result = await super().set_value(modbus, mqtt, index, source, handler)
         else:
-            logging.warning(f"{self.name} - Ignored attempt to set value to {value}: Not a valid mode")
+            logging.warning(f"{self.name} - Ignored attempt to set value to '{value}': Not a valid mode")
         if result:
             pass
         return result
